'''
Copyright (C) 2015 by the University of Southern California

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License,
version 2, as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

ip_iterator.py
-
Simple iterator over IPv4 address space
Adapted from ANT/lander_code/icmptrain/ip_iterator.cc
'''

class IP_Iterator:
    IP_ADDRESS_BITS = 32

    @staticmethod
    def reverse_bitstring(b, nbits):
        return int(bin(b)[2:].zfill(nbits)[::-1], 2)
    def __init__(self, total_bits=IP_ADDRESS_BITS, prefix=0, prefix_bits=0, nonce=0):
        self.total_bits_ = total_bits
        self.prefix_bits_ = prefix_bits
        self.suffix_bits_ = total_bits - prefix_bits
        self.prefix_mask_ = (2**prefix_bits - 1) << self.suffix_bits_
        self.suffix_mask_ = 2**self.suffix_bits_ - 1
        self.prefix_ = prefix & self.prefix_mask_
        self.nonce_ = nonce
        self.count_ = 0
        self.done_ = False
    def __iter__(self):
        return self
    def done(self):
        return self.done_
    def reset(self):
        self.done_ = False
        self.count_ = 0
    def next(self):
        if self.done_:
            raise StopIteration()
        x_pre = IP_Iterator.reverse_bitstring(self.count_, self.suffix_bits_) | self.prefix_
        supernet_nonce = x_pre >> 8
        x = x_pre ^ ((supernet_nonce ^ self.nonce_) & self.suffix_mask_)
        old_count = self.count_
        self.count_ = (self.count_ + 1)&self.suffix_mask_
        self.done_ = (self.count_ < old_count)
        return x

def num_to_ip(num):
    '''Takes an decimal representation of an IP and converts to a string'''
    return ".".join([str((num>>((3-i)*8))&0xFF) for i in range(0,4)])

#testing
if __name__ == "__main__":
    # 128.9/24
    # addrs = IP_Iterator(32, 0x80090000, 24, 0)
    # 128.9/16
    addrs = IP_Iterator(32, 0x80090000, 16, 0)
    # 128.9/8
    # addrs = IP_Iterator(32, 0x80090000, 8, 0)
    # Entire IPv4
    # addrs = IP_Iterator()
    ips = 0
    for ip in addrs:
        print num_to_ip(ip)
        ips += 1
    print 'iterated through {:d} ips'.format(ips)
