#!/bin/bash
# Copyright (C) 2015 by the University of Southern California
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# start_rdns_pass1.sh
# -
# A script for starting the rdns process accross multiple instances.
# Adapted from ANT/lander_code/icmptrain/ip_iterator.cc

MYDIR=$(dirname $0)
HOSTS=servers.txt
MODE=0 #first pass

cd $MYDIR
#files that we need on the remote system
export MYFILES=$(echo *.py)

TOTAL_JOBS=100 #should be roughly the same as number of cores

start_process() {
    jobs=$1
    instance=$2

    MODE=1 #xxx
    OUTPUTDIR=/users/$USER/rdns
    [ -d $OUTPUTDIR ] || mkdir -p $OUTPUTDIR
    cd /tmp/
    cp $MYFILES $OUTPUTDIR/

    cd $OUTPUTDIR
    BASE=pass_$MODE.$instance
    OUTPUT_MAIN=$BASE.fsdb
    TIMEOUTS=$BASE.timeouts.txt

    #xxx
    cmd="python2.7 rdns.py -i $MODE --total_instances=$jobs --current_instance=$instance --timeoutfile=$TIMEOUTS 128 >$OUTPUT_MAIN"
    echo "$(hostname): RUNNING $cmd"
    eval $cmd
}
export -f start_process

BFOPTS=
for f in $MYFILES; do
    BFOPTS="$BFOPTS --bf $f"
done
parallel \
    --no-notice \
    --workdir /tmp \
    $BFOPTS \
    --sshloginfile=$HOSTS \
    --env MYFILES \
    --env start_process \
    -P "100%" \
    "start_process $TOTAL_JOBS {}" ::: $( seq 0 $((TOTAL_JOBS-1)) )

#parallel --sshloginfile=$HOSTS --env MYDIR --env start_process -P "100%" "start_process $TOTAL_JOBS" ::: `seq 0 $TOTAL_JOBS`
#parallel start_process $TOTAL_JOBS ::: `seq 0 $TOTAL_JOBS`
